package app.services;

import app.entities.LecturerEntity;
import app.entities.StudentEntity;
import app.repositories.LecturerEntityRepository;
import app.repositories.StudentEntityRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

@Component
public class AuthenticationService {

    @Autowired
    private StudentEntityRepository studentEntityRepository;

    @Autowired
    private LecturerEntityRepository lecturerEntityRepository;

    public Map<String,String> authenticate(Map<String,String> request) {
        Map<String,String> response = new HashMap<>();
        if(request.get("username") != null && request.get("password") != null) {
            Optional<StudentEntity> student = studentEntityRepository.findByUsername(request.get("username"));
            Optional<LecturerEntity> lecturer = lecturerEntityRepository.findByUsername(request.get("username"));

            if (student.isPresent()) {
                if (compare(request.get("password"), student.get().getPassword())) {
                    response.put("auth", "true");
                    response.put("lec", "false");
                    return response;
                }
            }

            if (lecturer.isPresent()) {
                if (compare(request.get("password"), lecturer.get().getPassword())) {
                    response.put("auth", "true");
                    response.put("lec", "true");
                    return response;
                }
            }
        }
        response.put("auth","false");
        response.put("lec","false");
        return response;
    }

    private Boolean compare(String userHash, String dbHash) {
        BCryptPasswordEncoder encoder = new BCryptPasswordEncoder();
        return encoder.matches(userHash, dbHash);
    }

}
